#!/usr/bin/env bash
# ------------------------------------------------------------------------------
# This file profiles the provided binary on execution for the set platform.
# ------------------------------------------------------------------------------

# The BASEDIR is the PWD of profile.sh, regardless of the callers location.
export BASEDIR=$(cd -- "$( dirname -- "${BASH_SOURCE[0]}" )" &> /dev/null && pwd)

# Source helpers for colored prints.
source $BASEDIR/../helpers.sh


# Check if PLATFORM is valid.
function platformCheck {
    IFS="|"
    options=("lumi${IFS}mn5")

    if [[ -z "${PLATFORM}" ]]; then
        fatal "No platform selected. Options: [ lumi | mn5 ]"
        exit $?
    elif [[ ! "${IFS}${options[*]}${IFS}" =~ "${IFS}${PLATFORM}${IFS}" ]]; then
        fatal "Platform '${PLATFORM}' unknown. Options: [ lumi | mn5 ]"
        exit $?
    fi
}

# Profile binary on Lumi.
function profile_lumi {
    # Create folder for results.
    RESULTS=$BASEDIR/res-$1
    mkdir -p $RESULTS

    # Set runtime variables.
    export OMP_NUM_THREADS=1
    export MPICH_GPU_SUPPORT_ENABLED=1
    ROCMFILES="$RESULTS/rocprof.csv"

    # Profile the provided binary.
    info "[$(date +'%H:%M:%S')]  Executing: $1"
    rocprof --hip-trace --roctx-trace -o $ROCMFILES $BASEDIR/../bin/$1
    info "[$(date +'%H:%M:%S')]  Finished executing $1"
}

# Profile binary on MareNostrum 5.
function profile_mn5 {
    # TODO: Implement.
    echo "To be created.."
}

# Entry function.
function main {
    # Check if platform is set correctly.
    platformCheck

    # Check if provided binary exists.
    command -v $BASEDIR/../bin/$1 >/dev/null @>&1 || \
        fatal "Binary '$1' is not in bin folder."

    # Load modules.
    source $BASEDIR/../load_modules.sh

    # Profile provided binary on set platform.
    if [ "$PLATFORM" == "lumi" ]; then
        profile_lumi $1
        # sbatch profile_lumi.sh $1
    elif [ "$PLATFORM" == "mn5" ]; then
        profile_mn5 $1
    fi
}

main "$@"
